//+------------------------------------------------------------------+
//|                                           T3_Momentum_Burst.mq4  |
//+------------------------------------------------------------------+
#property copyright ""
#property link      ""

#property indicator_separate_window
#property indicator_buffers 1
#property indicator_color1  DeepSkyBlue
#property indicator_width1  2
#property indicator_level1  0

//
//
//
//
//

extern string TimeFrame      = "Current time frame";
extern int    Mom_Period     = 1;
extern int    T3_period      = 28;
extern double T3_hot         = 0.618;
extern int    T3_calculation = 2;
extern bool   Interpolate    = true;

//
//
//
//
//

double MapBuffer[];
double c1,c2,c3,c4,alpha;

//
//
//
//
//

int    timeFrame;
string IndicatorFileName;
bool   ReturningBars;
bool   CalculatingT3m;

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int init()
{
    SetIndexBuffer(0,MapBuffer);

   //
   //
   //
   //
   //
   
      double a  = T3_hot;
             c1 = -a*a*a;
             c2 =  3*(a*a+a*a*a);
             c3 = -3*(2*a*a+a+a*a*a);
             c4 = 1+3*a+a*a*a+3*a*a;

      T3_period = MathMax(1,T3_period);
      switch (T3_calculation)
      {
         case 0:  alpha = 2.0/(1.0 +   T3_period);                              break;    // original Tillson way
         case 1:  alpha = 2.0/(2.0 +  (T3_period-1.0)/2.0);                     break;    // Fulks / Matulich way
         default: alpha = 2.0/(2.0 + 0.5*(MathMax(1.0,1.0 + (0.5*(T3_period-1.0)))-1.0)); // undefined, kept for compability
      }         

      //
      //
      //
      //
      //
         
      IndicatorFileName = WindowExpertName();
      CalculatingT3m    = (TimeFrame=="CalculateT3m"); if (CalculatingT3m) return(0);
      ReturningBars     = (TimeFrame=="returnBars");   if (ReturningBars)  return(0);
      timeFrame         = stringToTimeFrame(TimeFrame);

   //
   //
   //
   //
   //
   
   IndicatorShortName("T3_Momentum_Burst "+timeFrameToString(timeFrame)+" ("+T3_period+")");
   IndicatorDigits(Digits);
   return(0);
}


//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int start()
{
   int counted_bars=IndicatorCounted();
   int i,limit;

   if(counted_bars<0) return(-1);
   if(counted_bars>0) counted_bars--;
         limit = MathMin(Bars-counted_bars,Bars-1);
         if (ReturningBars) { MapBuffer[0] = limit+1; return(0); }

   //
   //
   //
   //
   //
   
   if (CalculatingT3m || timeFrame==Period())
   {
      for(i=limit; i>=0; i--) MapBuffer[i]=iT3(iMomentum(NULL,0,Mom_Period,PRICE_CLOSE,i)-iMomentum(NULL,0,Mom_Period,PRICE_CLOSE,i+1),i);
      return(0);
   }
   
   
   //
   //
   //
   //
   //
         
   limit = MathMax(limit,MathMin(Bars,iCustom(NULL,timeFrame,IndicatorFileName,"returnBars",0,0)*timeFrame/Period()));
   for (i=limit;i>=0;i--)
   {
      int y = iBarShift(NULL,timeFrame,Time[i]);
         MapBuffer[i] = iCustom(NULL,timeFrame,IndicatorFileName,"CalculateT3m",Mom_Period,T3_period,T3_hot,T3_calculation,0,y);

         //
         //
         //
         //
         //
      
         if (timeFrame <= Period() || y==iBarShift(NULL,timeFrame,Time[i-1])) continue;
         if (!Interpolate) continue;

         //
         //
         //
         //
         //

         datetime time = iTime(NULL,timeFrame,y);
            for(int n = 1; i+n < Bars && Time[i+n] >= time; n++) continue;	
            double factor = 1.0 / n;
            for(int k = 1; k < n; k++)
               MapBuffer[i+k] = k*factor*MapBuffer[i+n] + (1.0-k*factor)*MapBuffer[i];
   }

   //
   //
   //
   //
   //
      
   return(0);
}


//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

double t3Values[][6];
double iT3(double price,int i,int s=0)
{
   if (ArrayRange(t3Values,0) != Bars) ArrayResize(t3Values,Bars);

   int r = Bars-i-1;
   if (r < 2)
      {
         t3Values[r][s+0] = price;
         t3Values[r][s+1] = price;
         t3Values[r][s+2] = price;
         t3Values[r][s+3] = price;
         t3Values[r][s+4] = price;
         t3Values[r][s+5] = price;
      }
   else
      {
         t3Values[r][s+0] = t3Values[r-1][s+0]+alpha*(price           -t3Values[r-1][s+0]);
         t3Values[r][s+1] = t3Values[r-1][s+1]+alpha*(t3Values[r][s+0]-t3Values[r-1][s+1]);
         t3Values[r][s+2] = t3Values[r-1][s+2]+alpha*(t3Values[r][s+1]-t3Values[r-1][s+2]);
         t3Values[r][s+3] = t3Values[r-1][s+3]+alpha*(t3Values[r][s+2]-t3Values[r-1][s+3]);
         t3Values[r][s+4] = t3Values[r-1][s+4]+alpha*(t3Values[r][s+3]-t3Values[r-1][s+4]);
         t3Values[r][s+5] = t3Values[r-1][s+5]+alpha*(t3Values[r][s+4]-t3Values[r-1][s+5]);
      }
   return(c1*t3Values[r][s+5] + c2*t3Values[r][s+4] + c3*t3Values[r][s+3] + c4*t3Values[r][s+2]);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = StringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}
string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//
//
//

string StringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int tchar = StringGetChar(s, length);
         if((tchar > 96 && tchar < 123) || (tchar > 223 && tchar < 256))
                     s = StringSetChar(s, length, tchar - 32);
         else if(tchar > -33 && tchar < 0)
                     s = StringSetChar(s, length, tchar + 224);
   }
   return(s);
}